@@ -97,7 +97,8 @@ class Manager(utils.HookableMixin):
         # pair
         username = utils.env('OS_USERNAME', 'NOVA_USERNAME')
         url = utils.env('OS_URL', 'NOVA_URL')
-        uniqifier = hashlib.md5(username + url).hexdigest()
+        uniqifier = hashlib.md5(username.encode('utf-8') +
+                                url.encode('utf-8')).hexdigest()
 
         cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))
 
@@ -201,13 +202,24 @@ class ManagerWithFind(Manager):
         searches = kwargs.items()
 
         detailed = True
-        if 'detailed' in inspect.getargspec(self.list).args:
+        list_kwargs = {}
+
+        list_argspec = inspect.getargspec(self.list)
+        if 'detailed' in list_argspec.args:
             detailed = ("human_id" not in kwargs and
                         "name" not in kwargs and
                         "display_name" not in kwargs)
-            listing = self.list(detailed=detailed)
-        else:
-            listing = self.list()
+            list_kwargs['detailed'] = detailed
+
+        if 'is_public' in list_argspec.args and 'is_public' in kwargs:
+            is_public = kwargs['is_public']
+            list_kwargs['is_public'] = is_public
+            if is_public is None:
+                tmp_kwargs = kwargs.copy()
+                del tmp_kwargs['is_public']
+                searches = tmp_kwargs.items()
+
+        listing = self.list(**list_kwargs)
 
         for obj in listing:
             try:
@@ -225,13 +237,45 @@ class ManagerWithFind(Manager):
 
 class BootingManagerWithFind(ManagerWithFind):
     """Like a `ManagerWithFind`, but has the ability to boot servers."""
+
+    def _parse_block_device_mapping(self, block_device_mapping):
+        bdm = []
+
+        for device_name, mapping in six.iteritems(block_device_mapping):
+            #
+            # The mapping is in the format:
+            # <id>:[<type>]:[<size(GB)>]:[<delete_on_terminate>]
+            #
+            bdm_dict = {'device_name': device_name}
+
+            mapping_parts = mapping.split(':')
+            source_id = mapping_parts[0]
+            if len(mapping_parts) == 1:
+                bdm_dict['volume_id'] = source_id
+
+            elif len(mapping_parts) > 1:
+                source_type = mapping_parts[1]
+                if source_type.startswith('snap'):
+                    bdm_dict['snapshot_id'] = source_id
+                else:
+                    bdm_dict['volume_id'] = source_id
+
+            if len(mapping_parts) > 2 and mapping_parts[2]:
+                bdm_dict['volume_size'] = str(int(mapping_parts[2]))
+
+            if len(mapping_parts) > 3:
+                bdm_dict['delete_on_termination'] = mapping_parts[3]
+
+            bdm.append(bdm_dict)
+        return bdm
+
     def _boot(self, resource_url, response_key, name, image, flavor,
               meta=None, files=None, userdata=None,
               reservation_id=None, return_raw=False, min_count=None,
               max_count=None, security_groups=None, key_name=None,
-              availability_zone=None, block_device_mapping=None, nics=None,
-              scheduler_hints=None, config_drive=None, admin_pass=None,
-              disk_config=None, **kwargs):
+              availability_zone=None, block_device_mapping=None,
+              block_device_mapping_v2=None, nics=None, scheduler_hints=None,
+              config_drive=None, admin_pass=None, disk_config=None, **kwargs):
         """
         Create (boot) a new server.
 
@@ -256,6 +300,8 @@ class BootingManagerWithFind(ManagerWithFind):
                                   placement.
         :param block_device_mapping: A dict of block device mappings for this
                                      server.
+        :param block_device_mapping_v2: A dict of block device mappings V2 for
+                                        this server.
         :param nics:  (optional extension) an ordered list of nics to be
                       added to this server, with information about
                       connected networks, fixed ips, etc.
@@ -322,30 +368,17 @@ class BootingManagerWithFind(ManagerWithFind):
 
         # Block device mappings are passed as a list of dictionaries
         if block_device_mapping:
-            bdm = body['server']['block_device_mapping'] = []
-            for device_name, mapping in block_device_mapping.items():
-                #
-                # The mapping is in the format:
-                # <id>:[<type>]:[<size(GB)>]:[<delete_on_terminate>]
-                #
-                bdm_dict = {'device_name': device_name}
+            body['server']['block_device_mapping'] = \
+                    self._parse_block_device_mapping(block_device_mapping)
+        elif block_device_mapping_v2:
+            # Append the image to the list only if we have new style BDMs
+            if image:
+                bdm_dict = {'uuid': image.id, 'source_type': 'image',
+                            'destination_type': 'local', 'boot_index': 0,
+                            'delete_on_termination': True}
+                block_device_mapping_v2.insert(0, bdm_dict)
 
-                mapping_parts = mapping.split(':')
-                id = mapping_parts[0]
-                if len(mapping_parts) == 1:
-                    bdm_dict['volume_id'] = id
-                if len(mapping_parts) > 1:
-                    type = mapping_parts[1]
-                    if type.startswith('snap'):
-                        bdm_dict['snapshot_id'] = id
-                    else:
-                        bdm_dict['volume_id'] = id
-                if len(mapping_parts) > 2:
-                    if mapping_parts[2]:
-                        bdm_dict['volume_size'] = str(int(mapping_parts[2]))
-                if len(mapping_parts) > 3:
-                    bdm_dict['delete_on_termination'] = mapping_parts[3]
-                bdm.append(bdm_dict)
+            body['server']['block_device_mapping_v2'] = block_device_mapping_v2
 
         if nics is not None:
             # NOTE(tr3buchet): nics can be an empty list
