@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-import StringIO
-
 import mock
 import six
 
@@ -26,6 +24,12 @@ class ServersTest(utils.TestCase):
         cs.assert_called('GET', '/servers')
         [self.assertTrue(isinstance(s, servers.Server)) for s in sl]
 
+    def test_list_servers_with_marker_limit(self):
+        sl = cs.servers.list(marker=1234, limit=2)
+        cs.assert_called('GET', '/servers/detail?marker=1234&limit=2')
+        for s in sl:
+            self.assertTrue(isinstance(s, servers.Server))
+
     def test_get_server_details(self):
         s = cs.servers.get(1234)
         cs.assert_called('GET', '/servers/1234')
@@ -38,7 +42,7 @@ class ServersTest(utils.TestCase):
         s2 = cs.servers.list(detailed=True)[0]
         self.assertNotEquals(s1._info, s2._info)
         s1.get()
-        self.assertEquals(s1._info, s2._info)
+        self.assertEqual(s1._info, s2._info)
 
     def test_create_server(self):
         s = cs.servers.create(
@@ -50,7 +54,7 @@ class ServersTest(utils.TestCase):
             key_name="fakekey",
             files={
                 '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': StringIO.StringIO('data'),   # a stream
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
             }
         )
         cs.assert_called('POST', '/servers')
@@ -94,10 +98,10 @@ class ServersTest(utils.TestCase):
             image=1,
             flavor=1,
             meta={'foo': 'bar'},
-            userdata=StringIO.StringIO('hello moto'),
+            userdata=six.StringIO('hello moto'),
             files={
                 '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': StringIO.StringIO('data'),   # a stream
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
         )
         cs.assert_called('POST', '/servers')
@@ -113,7 +117,7 @@ class ServersTest(utils.TestCase):
             key_name="fakekey",
             files={
                 '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': StringIO.StringIO('data'),   # a stream
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
         )
         cs.assert_called('POST', '/servers')
@@ -129,7 +133,7 @@ class ServersTest(utils.TestCase):
             key_name="fakekey",
             files={
                 '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': StringIO.StringIO('data'),   # a stream
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
             },
         )
         cs.assert_called('POST', '/servers')
@@ -501,6 +505,11 @@ class ServersTest(utils.TestCase):
         cs.servers.remove_security_group(s, 'oldsg')
         cs.assert_called('POST', '/servers/1234/action')
 
+    def test_list_security_group(self):
+        s = cs.servers.get(1234)
+        s.list_security_group()
+        cs.assert_called('GET', '/servers/1234/os-security-groups')
+
     def test_evacuate(self):
         s = cs.servers.get(1234)
         s.evacuate('fake_target_host', 'True')
