@@ -88,7 +88,9 @@ def get_resource_manager_extra_kwargs(f, args, allow_conflicts=False):
         conflicting_keys = set(hook_kwargs.keys()) & set(extra_kwargs.keys())
         if conflicting_keys and not allow_conflicts:
             raise Exception("Hook '%(hook_name)s' is attempting to redefine"
-                            " attributes '%(conflicting_keys)s'" % locals())
+                    " attributes '%(conflicting_keys)s'" %
+                    {'hook_name': hook_name,
+                        'conflicting_keys': conflicting_keys})
 
         extra_kwargs.update(hook_kwargs)
 
@@ -181,7 +183,7 @@ def print_dict(d, dict_property="Property", dict_value="Value", wrap=0):
             v = textwrap.fill(str(v), wrap)
         # if value has a newline, add in multiple rows
         # e.g. fault with stacktrace
-        if v and isinstance(v, basestring) and r'\n' in v:
+        if v and isinstance(v, six.string_types) and r'\n' in v:
             lines = v.strip().split(r'\n')
             col1 = k
             for line in lines:
@@ -192,7 +194,7 @@ def print_dict(d, dict_property="Property", dict_value="Value", wrap=0):
     print(strutils.safe_encode(pt.get_string()))
 
 
-def find_resource(manager, name_or_id):
+def find_resource(manager, name_or_id, **find_args):
     """Helper for the _find_* methods."""
     # first try to get entity as integer id
     try:
@@ -216,7 +218,7 @@ def find_resource(manager, name_or_id):
 
     try:
         try:
-            return manager.find(human_id=name_or_id)
+            return manager.find(human_id=name_or_id, **find_args)
         except exceptions.NotFound:
             pass
 
@@ -225,6 +227,7 @@ def find_resource(manager, name_or_id):
             resource = getattr(manager, 'resource_class', None)
             name_attr = resource.NAME_ATTR if resource else 'name'
             kwargs = {name_attr: name_or_id}
+            kwargs.update(find_args)
             return manager.find(**kwargs)
         except exceptions.NotFound:
             msg = "No %s with a name or ID of '%s' exists." % \
@@ -339,9 +342,10 @@ def slugify(value):
     From Django's "django/template/defaultfilters.py".
     """
     import unicodedata
-    if not isinstance(value, unicode):
+    if not isinstance(value, six.text_type):
         value = six.text_type(value)
-    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
+    value = unicodedata.normalize('NFKD', value).encode('ascii',
+                    'ignore').decode("ascii")
     value = six.text_type(_slugify_strip_re.sub('', value).strip().lower())
     return _slugify_hyphenate_re.sub('-', value)
 
