@@ -2,12 +2,10 @@
 """
 Flavor interface.
 """
-import urllib
-
 from novaclient import base
 from novaclient import exceptions
 from novaclient import utils
-from novaclient.openstack.common import uuidutils
+from novaclient.openstack.common.py3kcompat import urlutils
 
 
 class Flavor(base.Resource):
@@ -70,6 +68,12 @@ class Flavor(base.Resource):
                             "/flavors/%s/os-extra_specs/%s" % (
                             base.getid(self), k))
 
+    def delete(self):
+        """
+        Delete this flavor.
+        """
+        self.manager.delete(self)
+
 
 class FlavorManager(base.ManagerWithFind):
     """
@@ -90,7 +94,7 @@ class FlavorManager(base.ManagerWithFind):
         # and flavors from their own projects only.
         if not is_public:
             qparams['is_public'] = is_public
-        query_string = "?%s" % urllib.urlencode(qparams) if qparams else ""
+        query_string = "?%s" % urlutils.urlencode(qparams) if qparams else ""
 
         detail = ""
         if detailed:
@@ -148,12 +152,6 @@ class FlavorManager(base.ManagerWithFind):
 
         if flavorid == "auto":
             flavorid = None
-        elif not uuidutils.is_uuid_like(flavorid):
-            try:
-                flavorid = int(flavorid)
-            except (TypeError, ValueError):
-                raise exceptions.CommandError("Flavor ID must be an integer "
-                                              "or a UUID or auto.")
 
         try:
             swap = int(swap)
