@@ -19,13 +19,12 @@
 Server interface.
 """
 
-import urllib
-
 import six
 
 from novaclient import base
 from novaclient import crypto
-
+from novaclient.openstack.common.py3kcompat import urlutils
+from novaclient.v1_1.security_groups import SecurityGroup
 
 REBOOT_SOFT, REBOOT_HARD = 'SOFT', 'HARD'
 
@@ -318,10 +317,16 @@ class Server(base.Resource):
 
     def remove_security_group(self, security_group):
         """
-        Remova a security group from an instance.
+        Remove a security group from an instance.
         """
         self.manager.remove_security_group(self, security_group)
 
+    def list_security_group(self):
+        """
+        List security group(s) of an instance.
+        """
+        return self.manager.list_security_group(self)
+
     def evacuate(self, host, on_shared_storage, password=None):
         """
         Evacuate an instance from failed host to specified host.
@@ -364,12 +369,15 @@ class ServerManager(base.BootingManagerWithFind):
         """
         return self._get("/servers/%s" % base.getid(server), "server")
 
-    def list(self, detailed=True, search_opts=None):
+    def list(self, detailed=True, search_opts=None, marker=None, limit=None):
         """
         Get a list of servers.
-        Optional detailed returns details server info.
-        Optional reservation_id only returns instances with that
-        reservation_id.
+
+        :param detailed: Whether to return detailed server info (optional).
+        :param search_opts: Search options to filter out servers (optional).
+        :param marker: Begin returning servers that appear later in the server
+                       list than that represented by this server id (optional).
+        :param limit: Maximum number of servers to return (optional).
 
         :rtype: list of :class:`Server`
         """
@@ -382,7 +390,13 @@ class ServerManager(base.BootingManagerWithFind):
             if val:
                 qparams[opt] = val
 
-        query_string = "?%s" % urllib.urlencode(qparams) if qparams else ""
+        if marker:
+            qparams['marker'] = marker
+
+        if limit:
+            qparams['limit'] = limit
+
+        query_string = "?%s" % urlutils.urlencode(qparams) if qparams else ""
 
         detail = ""
         if detailed:
@@ -463,7 +477,7 @@ class ServerManager(base.BootingManagerWithFind):
         """
         Get password for an instance
 
-        Requires that openssl in installed and in the path
+        Requires that openssl is installed and in the path
 
         :param server: The :class:`Server` (or its ID) to add an IP to.
         :param private_key: The private key to decrypt password
@@ -569,7 +583,8 @@ class ServerManager(base.BootingManagerWithFind):
                reservation_id=None, min_count=None,
                max_count=None, security_groups=None, userdata=None,
                key_name=None, availability_zone=None,
-               block_device_mapping=None, nics=None, scheduler_hints=None,
+               block_device_mapping=None, block_device_mapping_v2=None,
+               nics=None, scheduler_hints=None,
                config_drive=None, disk_config=None, **kwargs):
         # TODO(anthony): indicate in doc string if param is an extension
         # and/or optional
@@ -597,6 +612,8 @@ class ServerManager(base.BootingManagerWithFind):
                                   placement.
         :param block_device_mapping: (optional extension) A dict of block
                       device mappings for this server.
+        :param block_device_mapping_v2: (optional extension) A dict of block
+                      device mappings for this server.
         :param nics:  (optional extension) an ordered list of nics to be
                       added to this server, with information about
                       connected networks, fixed ips, port etc.
@@ -628,6 +645,9 @@ class ServerManager(base.BootingManagerWithFind):
         if block_device_mapping:
             resource_url = "/os-volumes_boot"
             boot_kwargs['block_device_mapping'] = block_device_mapping
+        elif block_device_mapping_v2:
+            resource_url = "/os-volumes_boot"
+            boot_kwargs['block_device_mapping_v2'] = block_device_mapping_v2
         else:
             resource_url = "/servers"
         if nics:
@@ -834,24 +854,34 @@ class ServerManager(base.BootingManagerWithFind):
 
     def add_security_group(self, server, security_group):
         """
-        Add a Security Group to a instance
+        Add a Security Group to an instance
 
         :param server: ID of the instance.
-        :param security_grou: The name of security group to add.
+        :param security_group: The name of security group to add.
 
         """
         self._action('addSecurityGroup', server, {'name': security_group})
 
     def remove_security_group(self, server, security_group):
         """
-        Add a Security Group to a instance
+        Add a Security Group to an instance
 
         :param server: ID of the instance.
-        :param security_grou: The name of security group to remove.
+        :param security_group: The name of security group to remove.
 
         """
         self._action('removeSecurityGroup', server, {'name': security_group})
 
+    def list_security_group(self, server):
+        """
+        List Security Group(s) of an instance
+
+        :param server: ID of the instance.
+
+        """
+        return self._list('/servers/%s/os-security-groups' %
+                          base.getid(server), 'security_groups', SecurityGroup)
+
     def evacuate(self, server, host, on_shared_storage, password=None):
         """
         Evacuate a server instance.
