@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2012 IBM
+# Copyright 2012 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -51,11 +51,16 @@ class ServiceManager(base.ManagerWithFind):
         return self._list(url, "services")
 
     def enable(self, host, binary):
-        """Enable the service specified by hostname and binary"""
+        """Enable the service specified by hostname and binary."""
         body = {"host": host, "binary": binary}
         return self._update("/os-services/enable", body, "service")
 
     def disable(self, host, binary):
-        """Enable the service specified by hostname and binary"""
+        """Disable the service specified by hostname and binary."""
         body = {"host": host, "binary": binary}
         return self._update("/os-services/disable", body, "service")
+
+    def disable_log_reason(self, host, binary, reason):
+        """Disable the service with reason."""
+        body = {"host": host, "binary": binary, "disabled_reason": reason}
+        return self._update("/os-services/disable-log-reason", body, "service")
